////////////////////////////////////////////////////////////////////////////////////////
// VCO_MIDI
// Date: 15 Feb 2018
//
// MIDI control of AS10K through MCV Module: 
// - Added Trigger and second MIDI channel
// - Added and LFO by software (only sine wave for now)
// - Added velocity note CV
// - Added calibration and pitch CV
//
// Using this module we can control up to 4 CV signals from 0 to 5 volts with 12 bits
// of resolution. 
//
// By Bleeps and Chips (J.Cano & Y. Torroja)
// www.bleepsandchips.com
//
////////////////////////////////////////////////////////////////////////////////////////
// CC BY-SA 3.0 - https://creativecommons.org/licenses/by-sa/3.0/
// Feel free to use and modify this code in any way you want
// Would be great if you maintain our authorship somehow :)
////////////////////////////////////////////////////////////////////////////////////////
#include <Arduino.h>

// Stop VCO on note off: Comment following line if you do not want VCO to stop
// when MIDI note_off is received (thats the normal mode of analog synthesisers, VCO
// is always running)
//#define STOP_VCO_ON_NOTE_OFF 1

// If you want to use two MIDI channels (midi chanels 1 and 2, values 0 and 1) 
// uncomment following line
#define USE_MIDI_CHANNELS 1

// To simplify this development, this pins are fixed. Please, do not chage!
// GATE pin 
#define GATE_PIN 7    //-> To ENV Gate
#define TRIGGER_PIN 6 //-> To ENV Trigger

#if defined(TCCR5B) && defined(TCCR5A) && defined(TIMSK5)  // For atmega2560/1280
// CS0 pins 
#define CS0_PIN  53   //-> To MCV (CSO)
#define CS1_PIN  54   //-> To MCV (CS1)
// Measure pin
#define FREQ_PIN 48   //
#else
// CS pins 
#define CS0_PIN  10   //-> To MCV (CSO)
#define CS1_PIN  9    //-> To MCV (CS1)
// Measure pin
#define FREQ_PIN 8    //
#endif

// Calibration pin
#define  CAL_PIN 12     // High level on this pin starts calibration

//-----------------------------------------------------------------------------------------------------
// DAC modifications
//
// On Arduino UNO, Duecimila, Duemilanove...
// SCK -> PIN 13
// SDI -> PIN 11
// CS0 -> PIN 10
// CS1 -> PIN 9
//
// On Arduino MEGA...
// SCK -> PIN 52
// SDI -> PIN 51
// CS0 -> PIN 53
// CS1 -> PIN 54 // Not tested!
//
//-----------------------------------------------------------------------------------------------------

#define MIDI_NOTE_ON  0b10010000
#define MIDI_NOTE_OFF 0b10000000

void setup() {  
  // Initialize serial port to get midi messages
  Serial.begin(57600);

  // setup CV using DAC
  MCV_Setup(4096);

  // restore calibration
  loadCalibration();

  // setup calibration pin
  pinMode(CAL_PIN, INPUT);
  pinMode(GATE_PIN, OUTPUT);
  pinMode(TRIGGER_PIN, OUTPUT);
}

void loop() {  

  /////////////////////////
  // Calibration section
  // Check calibration pin
  if (digitalRead(CAL_PIN)) {
    // Step by step calibration (more precise, max error about 1%, takes quite a long time to execute)
    unsigned long iniTime = millis();
    MCV_Calibration(10);
    Serial.print(F("Elapsed time (s): "));
    Serial.println((millis() - iniTime)/1000);

    saveCalibration();    
  }

  /////////////////////////
  // MIDI playback section
  // We wait for incomming data
  if( Serial.available() != 0 )
  {

    byte readValue = Serial.read();
    byte channel;
    byte note;
    byte vel;
    // We check more significant bit is 1 (MIDI command)
    if( (readValue & 0x80) != 0 )
    {
      // Get MIDI channel
      #ifdef USE_MIDI_CHANNELS
        // We can only use MIDI channels 1 (value 0) and 2 (value 1)
        channel = (readValue & 0x0F);
      #endif

      // Esperamos dos bytes mas...
      while( Serial.available() == 0 );
      note =  Serial.read();
      while( Serial.available() == 0 );
      vel  = Serial.read();  
      // And we play note
      if( (readValue & 0b11110000) == MIDI_NOTE_ON ) // Check four most significant bits - NOTE ON
      {
        if (vel != 0)
        {
          MCV_SetNote(channel, note, vel);
          MCV_Set_Gate(1);
          MCV_Pulse_Trigger(1);
        } 
        else {
          #ifdef STOP_VCO_ON_NOTE_OFF
            MCV_Set(channel, 0);
          #endif
          MCV_Set_Gate(0);
        }
      } 
      if( (readValue & 0b11110000) == MIDI_NOTE_OFF ) // Check four most significant bits - NOTE OFF
      {
        #ifdef STOP_VCO_ON_NOTE_OFF
          MCV_Set(channel, 0);
        #endif
        MCV_Set_Gate(0);
      }
    }
  }
}











